/* 
   Tester for image drawing
   Copyright (C) 2006 Free Software Foundation, Inc.

   Written by: Fred Kiefer <FredKiefer@gmx.de>
     and Richard Frith-Macdonald <rfm@gnu.org>
   Created: February 2006

   This file is part of the GNUstep project.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330,
   Boston, MA 02111 USA.
*/
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSString.h>
#include <AppKit/AppKit.h>

#include "GNUstep.h"

static NSString *operations[] = {
  @"composite image",
  @"dissolve image",
  @"draw image",
  @"draw rep",
  @"copy bits"
};

@interface AppController: NSObject
{
  NSPopUpButton *button;
  NSWindow      *base;
  NSView        *normal;
  NSView        *flipped;
  NSView        *scaled;
  NSView        *rotated;
}
- (void) drawCurrentTestIn: (NSView*)sender;
- (void) redraw: (id)sender;
@end


@interface NormalView: NSView
{
}
@end

@implementation NormalView
- (void) drawRect: (NSRect)r
{
  AppController *d = [NSApp delegate];
  
 [d drawCurrentTestIn: self];
}
@end

@interface FlippedView: NormalView
{
}
@end


@implementation FlippedView

- (BOOL) isFlipped
{
  return YES;
}

@end

@implementation AppController

/* Mark our views as needing to be redisplayed because the drawing
 * operation has changed.
 * called by the popup menu selecting the drawing operation.
 */
- (void) redraw: (id)sender
{
  [[base contentView] setNeedsDisplayInRect: NSMakeRect(0, 0, 410, 410)];
}

/* Perform a drawing operation.
 * This is called by the sender's -drawRect: method,
 * so focus should already be locked on the sender.
 */
- (void) drawCurrentTestIn: (NSView*)sender
{
  NSImage *image = [NSImage imageNamed: @"GNUstep"];
  //NSLog(@"image %@ %@ %@", [image name], image, [image _resource]);
  NSPoint point = NSMakePoint(20, 20);
  NSString *title = [button titleOfSelectedItem];
  
  [[NSColor blueColor] set];
  NSRectFill([sender bounds]);

  NSLog(@"In '%@' point is %@ in super, %@ in window", title,
    NSStringFromPoint([sender convertPoint: point toView: [sender superview]]),
    NSStringFromPoint( [sender convertPoint: point toView: nil]));

  if ([title isEqualToString: @"composite image"] == YES)
    {
      [image compositeToPoint: point
                    operation: NSCompositeSourceOver
                     fraction: 0.8];
    }
  else if ([title isEqualToString: @"draw image"] == YES)
    {
      [image drawAtPoint: point
                fromRect: NSMakeRect(0,0,50,50)
               operation: NSCompositeSourceOver
                fraction: 0.8];
    }
  else if ([title isEqualToString: @"dissolve image"] == YES)
    {
      [image dissolveToPoint: point
                    fraction: 0.4];
    }
  else if ([title isEqualToString: @"draw rep"] == YES)
    {
      [[image bestRepresentationForDevice: nil] drawAtPoint: point];
    }
  else if ([title isEqualToString: @"copy bits"] == YES)
    {
      [image compositeToPoint: NSMakePoint(20,20)
                    operation: NSCompositeSourceOver
                     fraction: 0.8];
      if (sender == flipped)
        {
          NSCopyBits(0, NSMakeRect(0, 0, 50, 50), NSMakePoint(70,70));
        }
      else
        {
          NSCopyBits(0, NSMakeRect(20, 20, 50, 50), NSMakePoint(70,70));
        }
      [sender scrollRect: NSMakeRect(70, 70, 50, 50) by: NSMakeSize(-10, 10)];
    }
  else
    {
      NSLog(@"Unknown menu item '%@'", title);
    }
}


- (void) baseWindow
{ 
  unsigned      i = sizeof(operations) / sizeof(*operations);
  NSBox *box;
  NSView *cv;
  NSScrollView *sv;
  
  base = [[NSWindow alloc] initWithContentRect: NSMakeRect(200,200,410,450)
                           styleMask: NSTitledWindowMask | NSClosableWindowMask
                           | NSMiniaturizableWindowMask | NSResizableWindowMask
                           backing: NSBackingStoreBuffered
                           defer: NO];
  [base setTitle: @"Test window"];
  
  button = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(100,420,210,25)
                                      pullsDown: NO];
  while (i-- > 0)
    {
      [button insertItemWithTitle: operations[i] atIndex: 0];
      [[button itemAtIndex: 0] setTarget: self];
      [[button itemAtIndex: 0] setAction: @selector(redraw:)];
        }
  [button selectItemAtIndex: 0];

  cv = [base contentView];
  [cv addSubview: button];
  
  box = [[NSBox alloc] initWithFrame: NSMakeRect(0,210,200,200)];
  sv = [[NSScrollView alloc] initWithFrame: NSMakeRect(0,0,200,200)];
  [box setTitle: @"Normal"];
  [cv addSubview: box];
  normal = [[NormalView alloc] initWithFrame: [[box contentView] frame]];
  [sv setDocumentView: normal];
  [box setContentView: sv];
  
  box = [[NSBox alloc] initWithFrame: NSMakeRect(210,210,200,200)];
  sv = [[NSScrollView alloc] initWithFrame: NSMakeRect(0,0,200,200)];
  [box setTitle: @"Flipped"];
  [cv addSubview: box];
  flipped = [[FlippedView alloc] initWithFrame: [[box contentView] frame]];
  [sv setDocumentView: flipped];
  [box setContentView: sv];

  box = [[NSBox alloc] initWithFrame: NSMakeRect(0,0,200,200)];
  sv = [[NSScrollView alloc] initWithFrame: NSMakeRect(0,0,200,200)];
  [box setTitle: @"Scaled"];
  [cv addSubview: box];
  scaled = [[NormalView alloc] initWithFrame: [[box contentView] frame]];
  [sv setDocumentView: scaled];
  [box setContentView: sv];
  [scaled  scaleUnitSquareToSize: NSMakeSize(2.0,2.0)];
  
  
  box = [[NSBox alloc] initWithFrame: NSMakeRect(210,0,200,200)];
  sv = [[NSScrollView alloc] initWithFrame: NSMakeRect(0,0,200,200)];
  [box setTitle: @"Rotated"];
  [cv addSubview: box];
  rotated = [[NormalView alloc] initWithFrame: [[box contentView] frame]];
  [[base contentView] addSubview: rotated];
  [sv setDocumentView: rotated];
  [box setContentView: sv];
  [rotated translateOriginToPoint: NSMakePoint(50,50)];
  [rotated rotateByAngle: 45.0];
  
  [base makeKeyAndOrderFront: nil];
}


- (void) applicationDidFinishLaunching:(NSNotification *)notification
{
  NSMenu *menu = [NSMenu new];

  [menu addItemWithTitle: @"Quit"
        action: @selector(terminate:)
        keyEquivalent: @"q"];
  [NSApp setMainMenu: menu];
  RELEASE(menu);

  [self baseWindow];
}

@end

int main (int argc, const char *argv[])
{
  CREATE_AUTORELEASE_POOL(pool);
  id app;

  app = [NSApplication sharedApplication];
  [app setDelegate: [AppController new]];
  [app run];
  RELEASE(pool);
  return 0;
}
